/*
* Distributed Computational System
* Protocol API
*
* $Id: protocol.h,v 1.5 2001/11/29 07:06:17 kris Exp $
*/

#ifndef _PROTOCOL_H
#define _PROTOCOL_H

#include <string.h>	/* for memset */

/** Data Structures **/

typedef struct _command_t {
	unsigned short cmd;		/* command number */
	long len;			/* length of data */
	unsigned char *data;		/* command data */
} command_t;

/*
	IMPORTANT: when declaring a new variable of this type, or
	when preparing to re-use a previously declared variable,
	the variable should be re-initialized (zeroed).
	ALSO IMPORTANT: i changed this slightly, moving encoding
	to before cmd because otherwise gcc decides this struct
	needs 24 bytes, and not 20 (which makes reading into it
	awful funky).
*/
typedef struct _packet_buf_t {
	unsigned long bytes_read;	/* current number of bytes we've read */
	unsigned char version;		/* protocol version */
	unsigned char encoding;		/* data encoding flags */
	unsigned short cmd;		/* command */
	unsigned long checksum;		/* data checksum */
	long len;			/* length of data */
	unsigned char *data;		/* command data */
} packet_buf_t;

/** Definitions **/

/*
	useful way of constructing a command_t on the fly to pass
	as an argument to PR_Send
*/
#define MAKECMD(c,l,d) ((command_t){(c),(l),(d)})

/*
	a macro for getting a command_t from a completely read
	packet_buf_t
*/
#define GETCMD(p) ((command_t){(p).cmd,(p).len,(p).data})

/*
	zero the memory of a packet buffer
*/
#define INIT_PACKET_BUF(p) memset( &(p), 0, sizeof( (p) ) )

/*
	deallocate memory allocated for the data portion of a
	packet buffer (this memory is allocated inside of PR_Recv)
*/
#define FREE_PACKET_DATA(p) if( (p).data ) free( (p).data )
#define FREE_COMMAND_DATA(c) if( (c).data ) free( (c).data )

/** Function Prototypes **/

int PR_Send( int socket, unsigned short flags, command_t *command );

/*
	The return values for PR_Recv and PR_RecvBlocking are as
	follows:
		-1:	error
		0:	socket was disconnected before packet was
			completely read
		1:	no more data is currently available, but
			the socket is still connected (only
			PR_Recv)
		2:	the current packet has been completely
			read into the packet buffer, and should
			now be handled
*/
int PR_Recv( int socket, packet_buf_t *packet_buf );
int PR_RecvBlocking( int socket, command_t *cmd );

#endif /* _PROTOCOL_H */
